package main

/*
1. 算法清理可以用仿真云平台的调度系统
*/

import (
	"cicv-data-closedloop/amd64/dispatch_server/package/handler"
	"cicv-data-closedloop/amd64/dispatch_server/package/infra"
	commonHandler "cicv-data-closedloop/common/gin/handler"
	"cicv-data-closedloop/common/util"
	_ "embed"
	"github.com/gin-gonic/gin"
	"gopkg.in/yaml.v3"
	"os"
)

type ApplicationYamlStruct struct {
	ApplicationName string      `yaml:"application"`
	Web             WebStruct   `yaml:"web"`
	Redis           RedisStruct `yaml:"redis"`
	Oss             OssStruct   `yaml:"oss"`
	GpuNodeList     []GpuNode   `yaml:"gpu_node_list"`
}

type WebStruct struct {
	Port        string   `yaml:"port"`
	RoutePrefix string   `yaml:"route-prefix"`
	Token       string   `yaml:"token"`
	WhiteList   []string `yaml:"white-list"`
}

type RedisStruct struct {
	Addr     string `yaml:"addr"`
	Password string `yaml:"password"`
	Db       int    `yaml:"db"`
}
type OssStruct struct {
	IsUseCname      bool   `json:"is-use-cname"`
	Endpoint        string `json:"endpoint"`
	AccessKeyId     string `json:"accessKeyId"`
	AccessKeySecret string `json:"accessKeySecret"`
	BucketName      string `json:"bucketName"`
}

type GpuNode struct {
	Hostname    string `yaml:"hostname"`
	Ip          string `yaml:"ip"`
	Parallelism string `yaml:"parallelism"`
}

var (
	//go:embed application.yaml
	applicationYamlBytes []byte
	ApplicationYaml      ApplicationYamlStruct
)

func init() {
	// 1 解析YAML内容
	_ = yaml.Unmarshal(applicationYamlBytes, &ApplicationYaml)
	// 2 初始化 日志
	infra.InitLog("./log/", ApplicationYaml.ApplicationName)
	// 3 初始化 阿里云oss 客户端
	infra.InitOss(
		ApplicationYaml.Oss.IsUseCname,
		ApplicationYaml.Oss.Endpoint,
		ApplicationYaml.Oss.AccessKeyId,
		ApplicationYaml.Oss.AccessKeySecret,
		ApplicationYaml.Oss.BucketName,
	)
	// 4 初始化 Redis 客户端
	infra.InitRedisClient(
		ApplicationYaml.Redis.Addr,
		ApplicationYaml.Redis.Password,
		ApplicationYaml.Redis.Db,
	)
}

func main() {
	router := gin.Default()
	router.Use(commonHandler.ValidateHeaders())
	api := router.Group(ApplicationYaml.Web.RoutePrefix)
	api.POST("/start-project", handler.StartProject)
	err := router.Run(":" + ApplicationYaml.Web.Port)
	if err != nil {
		infra.GlobalLogger.Error("程序崩溃，监听端口 " + util.ToString(ApplicationYaml.Web.Port) + " 失败。")
		os.Exit(-1)
	}
}
